# -*- coding: utf-8 -*-
"""
Knowledge 模块
提供自进化知识库加载机制：两级目录扫描 + frontmatter 解析 + 关键词匹配生成索引。
"""
from .curator import KnowledgeCurator
from .loader import Knowledge, KnowledgeLoader, create_default_loader

__all__ = [
    "Knowledge",
    "KnowledgeLoader",
    "create_default_loader",
    "KnowledgeCurator",
]
