# -*- coding: utf-8 -*-
"""Version control services package."""
from .commit_attribution import (
    AuthorStats,
    BlameLine,
    CommitAttributionService,
    CommitInfo,
    get_default_service,
)

__all__ = [
    "CommitAttributionService",
    "BlameLine",
    "CommitInfo",
    "AuthorStats",
    "get_default_service",
]
