# -*- coding: utf-8 -*-
"""
Skills 模块
提供 Agent Skills 加载与注入机制（SKILL.md 格式）
"""
from .loader import Skill, SkillLoader, create_default_loader

__all__ = [
    "Skill",
    "SkillLoader",
    "create_default_loader",
]
