# -*- coding: utf-8 -*-
"""
命令模块
提供 Slash Commands 等用户快捷指令机制
"""
from .slash import SlashCommand, SlashCommandRegistry, create_default_registry

__all__ = [
    "SlashCommand",
    "SlashCommandRegistry",
    "create_default_registry",
]
