# -*- coding: utf-8 -*-
"""
MCP 模块
提供 Model Context Protocol 客户端能力，连接外部 MCP Server
"""
from .manager import MCPClientManager, ServerConfig, MCPServerType, create_default_manager

__all__ = [
    "MCPClientManager",
    "ServerConfig",
    "MCPServerType",
    "create_default_manager",
]
