# -*- coding: utf-8 -*-
"""
Hooks 模块
提供生命周期事件钩子机制（PreToolUse/PostToolUse/PreLLMRequest 等）
"""
from .runner import HookRunner, HookResult, create_default_runner

__all__ = [
    "HookRunner",
    "HookResult",
    "create_default_runner",
]
