# -*- coding: utf-8 -*-
"""abyss 配置包。

包含：
- 原生 Config（向后兼容，从 .legacy 重导出）
"""
# 向后兼容：原 `from abyss.config import Config, init_user_dirs` 等保持可用
from .legacy import (
    Config,
    CONFIG_DIR,
    CONFIG_FILE,
    SESSION_DIR,
    init_user_dirs,
)

__all__ = [
    "Config",
    "CONFIG_DIR",
    "CONFIG_FILE",
    "SESSION_DIR",
    "init_user_dirs",
]
